﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using InstrumentDrivers;

/// <summary>
/// Version history:
/// 
/// Version 1.0.0
/// - First Version created
/// </summary>
namespace rsnrpz_Continous_Average_2Sensors
{
  /// <summary>
  /// The example initiates two sessions, performs zeroing of both sensors and then starts multiple continuous average measurements.
  /// Zeroing and measurements are perform in two separate parallel threads to increase the speed.
  /// rsnrpz64.cs wrapper taken from rsnrpz driver 3.5.6.0 64-bit: "c:\Program Files\IVI Foundation\VISA\Win64\Include\rsnrpz64.cs"
  /// To build the 32-bit version, install the rsnrpz_vxipnp_3_5_6_0_32bit driver and replace the rsnrpz64.cs with its 32-bit equivalent:
  /// "c:\Program Files (x86)\IVI Foundation\VISA\WinNT\include\rsnrpz.cs"
  /// </summary>
  class Program
  {
    /// <summary>
    /// Function for measuring a single value in dBm
    /// </summary>
    /// <param name="sensor">sensor session</param>
    /// <param name="timeoutMilliseconds">measurement timeout</param>
    /// <returns>result in dBm</returns>
    static double Measure(rsnrpz sensor, int timeoutMilliseconds)
    {
      double resultWatt;
      sensor.Ext_InitAndWaitForMeasurementComplete(timeoutMilliseconds);
      sensor.meass_fetchMeasurement(1, out resultWatt); //fetch the result in Watts
      //Watt to dBm, for low and sometimes slightly negative values coerce to -200dBm
      double measValueDBM = resultWatt > 0 ? (10 * Math.Log(resultWatt) / Math.Log(10)) + 30.0 : -200;
      return measValueDBM;
    }
    
    static void Main(string[] args)
    {
      rsnrpz sensorOne = null;
      rsnrpz sensorTwo = null;
      string resourceNameOne = "USB::0x0aad::0x0139::101460";
      string resourceNameTwo = "USB::0x0aad::0x0023::102800";
      int avgResolution = 2; // Range 1 - 4, the higher resolution, the more stable results, but longer measurement time
      int measurementCount = 10;
            
      // Initiate the session sensorOne
      try
      {
        sensorOne = new rsnrpz(resourceNameOne, true, true);
      }
      catch (Exception ex)
      {
        Console.WriteLine("\n\nError initializing the sensorOne '{0}' Make sure the resource string is valid for your sensor.\nException message: {1}\n", resourceNameOne, ex.Message);
        Console.Write("Press any key to finish ...");
        Console.ReadKey();
        return;
      }

      // Initiate the session sensorTwo
      try
      {
        sensorTwo = new rsnrpz(resourceNameTwo, true, true);
      }
      catch (Exception ex)
      {
        Console.WriteLine("\n\nError initializing the sensorTwo '{0}' Make sure the resource string is valid for your sensor.\nException message: {1}\n", resourceNameTwo, ex.Message);
        Console.Write("Press any key to finish ...");
        Console.ReadKey();
        return;
      }
        
      try
      {
        // sensorOne settings
        Console.Write("\nApplying settings to sensorOne '{0}' ...", resourceNameOne);
        sensorOne.chan_mode(1, rsnrpzConstants.SensorModeContav); //Continuous average mode non-buffered
        sensorOne.chan_setCorrectionFrequency(1, 100E6); //correction frequency 100MHz
        sensorOne.avg_configureAvgAuto(1, avgResolution); // auto averaging with defined digits resolution
        sensorOne.trigger_setSource(1, rsnrpzConstants.TriggerSourceImmediate); //trigger immediate
        sensorOne.chan_setContAvSmoothingEnabled(1, false); //smoothing of the results using a low-pass filter
        Console.WriteLine(" finished.");

        // sensorTwo settings
        Console.Write("Applying settings to sensorTwo '{0}' ...", resourceNameTwo);
        sensorTwo.chan_mode(1, rsnrpzConstants.SensorModeContav); //Continuous average mode non-buffered
        sensorTwo.chan_setCorrectionFrequency(1, 100E6); //correction frequency 100MHz
        sensorTwo.avg_configureAvgAuto(1, avgResolution); // auto averaging with defined digits resolution
        sensorTwo.trigger_setSource(1, rsnrpzConstants.TriggerSourceImmediate); //trigger immediate
        sensorTwo.chan_setContAvSmoothingEnabled(1, false); //smoothing of the results using a low-pass filter
        Console.WriteLine(" finished.");
        
        Console.WriteLine("\nRemove any signal from sensor inputs ...\nZeroing boths sensors in two paralell threads ... ");
        // Parallel zeroing tasks start
        Parallel.Invoke(
          () => { Console.WriteLine("Sensor 1 zeroing started ... "); sensorOne.Ext_ZeroAndWaitForComplete(10000); Console.WriteLine("Sensor 1 zeroing finished"); },
          () => { Console.WriteLine("Sensor 2 zeroing started ... "); sensorTwo.Ext_ZeroAndWaitForComplete(10000); Console.WriteLine("Sensor 2 zeroing finished"); }
          );
        // End of parallel zeroing tasks, here the program waits for all the parallel tasks to finish

        Console.Write("\nApply signals to the sensor inputs and press any key to continue ...");
        Console.ReadKey();

        Console.WriteLine("\nMeasuring in two parallel tasks");
        // Parallel measurements start
        Parallel.Invoke(
            () => { for (int i = 0; i < measurementCount; i++) Console.WriteLine("Sensor 1 result no. {0}: {1:F3}", i+1, Measure(sensorOne, 3000)); },
            () => { for (int i = 0; i < measurementCount; i++) Console.WriteLine("  Sensor 2 result no. {0}: {1:F3}", i+1, Measure(sensorTwo, 3000)); }
          );
        // End of parallel tasks, here the program waits for all the parallel tasks to finish

      }

      catch (Exception ex)
      {
        Console.WriteLine("\nException occured: " + ex.Message);
      }
      finally
      {
        if (sensorOne != null) sensorOne.Dispose();
        if (sensorTwo != null) sensorTwo.Dispose();

        Console.WriteLine("\n\nPress any key to finish");
        Console.ReadKey();
      }
    }
  }

  /// <summary>
  /// rsnrpz Extension methods
  /// </summary>
  static class ExtensionMethods
  {
    /// <summary>
    /// Extension method for starting zeroing an dwaiting until the it is complete
    /// </summary>
    /// <param name="?"></param>
    /// <param name="timeoutMilliseconds">zeroing timeout in milliseconds</param>
    public static void Ext_ZeroAndWaitForComplete(this rsnrpz sensor, int timeoutMilliseconds)
    {
      System.DateTime tout = System.DateTime.Now.AddMilliseconds(timeoutMilliseconds);
      bool complete;
      sensor.chan_zero(1);
      do
      {
        sensor.chan_isZeroComplete(1, out complete);
          System.Threading.Thread.Sleep(100);
      } while (!complete && (System.DateTime.Now.CompareTo(tout) < 0));
    }

    /// <summary>
    /// Extension method for initiating a measurement and waiting until it is complete
    /// </summary>
    /// <param name="sensor"></param>
    /// <param name="timeoutMilliseconds"></param>
    public static void Ext_InitAndWaitForMeasurementComplete(this rsnrpz sensor, int timeoutMilliseconds)
    {
      System.DateTime tout = System.DateTime.Now.AddMilliseconds(timeoutMilliseconds);
      bool complete;
      sensor.chan_initiate(1);
      do
      {
        sensor.chan_isMeasurementComplete(1, out complete);
        System.Threading.Thread.Sleep(100);
      } while (!complete && (System.DateTime.Now.CompareTo(tout) < 0));
    }
  }
}