﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using InstrumentDrivers;

/// <summary>
/// Version history:
/// 
/// Version 1.0.0
/// - First Version created
/// </summary>
namespace rsnrpz_Get_Sensors_Info
{
    /// <summary>
    /// The example retrieves the information about all the available R&S NRP-Z and NRP-S/SN Powersensors (connected over USB)
    /// Then, as an example of use, it initiates two separate connections to two sensors and performs zeroing on both of them.
    /// rsnrpz64.cs wrapper taken from rsnrpz driver 3.5.6.0 64-bit: "c:\Program Files\IVI Foundation\VISA\Win64\Include\rsnrpz64.cs"
    /// To build the 32-bit version, install the rsnrpz_vxipnp_3_5_6_0_32bit driver and replace the rsnrpz64.cs with its 32-bit equivalent:
    /// "c:\Program Files (x86)\IVI Foundation\VISA\WinNT\include\rsnrpz.cs"
    /// </summary>
    class Program
    {
        static void Main(string[] args)
        {
            int sensorsCount = 0;
            var resourceNames = new List<string>();
            var resourceName = new StringBuilder(1000);
            var sensorType = new StringBuilder(1000);
            var sensorSN = new StringBuilder(1000);

            try
            {
                // To get an access to the methods GetSensorCount() and GetSensorInfo(), we need to create a valid rsnrpz object
                // We will find first available sensor and use it as an auxSensor, enumerate all available sensors and then destroy the auxSensor object.
                var auxSensor = new rsnrpz("USB::0xAAD::*?", true, true);

                auxSensor.GetSensorCount(out sensorsCount);
                Console.WriteLine("Number of sensors found: " + sensorsCount);

                for (int i = 1; i <= sensorsCount; i++)
                {
                    auxSensor.GetSensorInfo(i, resourceName, sensorType, sensorSN); //read sensor info
                    resourceNames.Add(resourceName.ToString());
                    Console.WriteLine("\n\nSensor number {0}:", i);
                    Console.WriteLine("Sensor Resource Name: " + resourceName);
                    Console.WriteLine("Sensor Type: " + sensorType);
                    Console.WriteLine("Sensor Serial Number: " + sensorSN);
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine("\n\nError initializing the dummy sensor. Make sure there is at least one sensor present.\nException message: {0}\n", ex.Message);
                Console.Write("Press any key to finish ...");
                Console.ReadKey();
                return;
            }

            // Assuming you have at least two sensors, open two independent sessions and always use channel parameter = 1:
            if (sensorsCount < 2)
            {
                Console.WriteLine("\n\nTo continue with opening two sessions, you need at least two sensors to be present\n");
                Console.Write("Press any key to finish ...");
                Console.ReadKey();
                return;
            }

            rsnrpz sensorOne = null;
            rsnrpz sensorTwo = null;

            // Initiate the session sensorOne
            try
            {
                sensorOne = new rsnrpz(resourceNames[0], true, true);
            }
            catch (Exception)
            {
                Console.WriteLine("\n\nError initializing the sensor {0}. Press any key to finish", resourceNames[0]);
                Console.ReadKey();
                return;
            }

            // Initiate the session sensorTwo
            try
            {
                sensorTwo = new rsnrpz(resourceNames[0], true, true);
            }
            catch (Exception)
            {
                Console.WriteLine("\n\nError initializing the sensor {0}. Press any key to finish", resourceNames[1]);
                Console.ReadKey();
                return;
            }

            try
            {
                Console.WriteLine("\nRemove any signal from sensor inputs ...\nZeroing boths sensors in two paralell threads ... ");
                // Parallel tasks start
                Parallel.Invoke(
                    () => { Console.WriteLine("Sensor 1 zeroing started ... "); sensorOne.Ext_ZeroAndWaitForComplete(10000); Console.WriteLine("Sensor 1 zeroing finished"); },
                    () => { Console.WriteLine("Sensor 2 zeroing started ... "); sensorTwo.Ext_ZeroAndWaitForComplete(10000); Console.WriteLine("Sensor 2 zeroing finished"); }
                ); 
                // End of parallel tasks, here the program waits for all the parallel tasks to finish
            }
            catch (Exception ex)
            {
                Console.WriteLine("\nException occured: " + ex.Message);
            }
            finally
            {
                sensorOne.Dispose();
                sensorTwo.Dispose();

                Console.WriteLine("\n\nPress any key to finish");
                Console.ReadKey();
            }
        }
    }


    /// <summary>
    /// rsnrpz Extension methods
    /// </summary>
    static class ExtensionMethods
    {
        /// <summary>
        /// Extension method for starting zeroing an dwaiting until the it is complete
        /// </summary>
        /// <param name="?"></param>
        /// <param name="timeoutMilliseconds">zeroing timeout in milliseconds</param>
        public static void Ext_ZeroAndWaitForComplete(this rsnrpz sensor, int timeoutMilliseconds)
        {
            System.DateTime tout = System.DateTime.Now.AddMilliseconds(timeoutMilliseconds);
            bool complete;
            sensor.chan_zero(1);
            do
            {
                sensor.chan_isZeroComplete(1, out complete);
                System.Threading.Thread.Sleep(100);
            } while (!complete && (System.DateTime.Now.CompareTo(tout) < 0));
        }
    }
}